==> bot4-0.0.8-py3-none-any.whl: bot4/protocol/protocolbase.py <==
import asyncio, zlib
from mcd.packets import Coder
from router.packetsrouter import PacketRouter, PacketHeandler, PacketStorage
from router.baserouter import BaseRouter, BaseHeandler
from buffer import BufferUnderrun, Buffer
from bot import botbase
from typesdata.v754 import VarInt
from exceptions import ProtocolError
from typing import Callable


def _add_callback(func_add: Callable):
    def _add(self: 'PortocolBase', event: BaseHeandler | str, func: Callable = None):
        def _w(func):
            if isinstance(event, str):
                getattr(self.router_packets, func_add.__name__)(event, func)
            elif isinstance(event, BaseHeandler):
                try:
                    getattr(self._table_routers[event.target_router], func_add.__name__)(event, func)
                except KeyError:
                    raise ProtocolError(f'router <{event.target_router.__name__}> unregistered')

        if func is None:
            return _w
        else:
            _w(func)
    return _add


class PortocolBase(asyncio.Protocol):
    router_packets: PacketRouter = None
    state: int
    unheandler = None

    def __init__(self, bot: 'botbase.BotBase',
                 packet_encoder_s: Coder,
                 router_packets: PacketRouter,
                 state: int):
        self.bot = bot
        
        self.loop = bot.loop
        self.recv_buff = bot.recv_buff
        self.cipher = bot.cipher
        self.buff_type = bot.buff_type
        self.logger = bot.logger
        
        self.state = state
        self.router_packets = router_packets
        self.packet_encoder = packet_encoder_s
        self._table_routers = {router_packets.__class__: router_packets}

        self._table_routers_names = {}
        self.table_routers_clear = self._table_routers.clear

    @_add_callback
    def on(self, event: BaseHeandler | str, func: Callable = None):
        ...

    @_add_callback
    def once(self, event: BaseHeandler | str, func: Callable = None):
        ...

    def reg_router(self, router: BaseRouter):
        if router.__class__ in self._table_routers:
            raise ProtocolError(f'router "{router.__class__.__name__}" is already registered')
        elif isinstance(router, BaseRouter):
            if self.router_packets.__class__ is router.__class__:
                self.router_packets = router
                # raise ProtocolError(f'router type <{router.__class__}> is not router_packets type <{self.router_packets.__class__}>')
            self._table_routers[router.__class__] = router
        else:
            raise ProtocolError(f'"router" <{router}> does not inherit "BaseRouter"')

    def reg_routers(self, *routers: BaseRouter):
        for r in routers:
            self.reg_router(r)

    def unreg_router(self, router: BaseRouter):
        if router.__class__ in self._table_routers:
            del self._table_routers[router.__class__]
            name = router.__class__.__name__.lower()
            if name in self._table_routers_names:
                del self._table_routers_names[name]

            if not self._table_routers:
                self.router_packets.clear()
                self.router_packets = None
            return True
        else:
            return False
    
    def run(self):
        self.bot.is_close = False
        # self.bot.state = self.state

    def switch(self):
        self.logger.debug(f'Switched protocol "{self.__class__.__name__}" state {self.state}')
        self.router_packets.clear()

    def connection_made(self, transport: asyncio.Transport) -> None:
        self.bot.transport = transport
        self.logger.info(f'Client connected to "{self.bot.ip}:{self.bot.port}"')

    async def asend_packet(self, *args):
        self.send_packet(*args)

    def connection_lost(self, exc: Exception | None) -> None:
        self.switch()
        self.bot._connection_lost(exc)

    def data_received(self, data: bytearray):
        self.recv_buff.add(self.cipher.decrypt(bytes(data)))
        
        if self.state != self.bot.state: self.logger.debug(f'{self.state} {self.bot.state}')
        while self.state == self.bot.state:
            self.recv_buff.save()
            try:
                buff: Buffer = self.unpack_packet(VarInt.decode(self.recv_buff))
                id_packet = VarInt.decode(buff)
                try:
                    self.router_packets.rout(id_packet, buff)
                except KeyError as e:
                    self.logger.critical(f'{e} state {self.state} protocol {self.__class__.__name__}')
            
            except BufferUnderrun:
                self.recv_buff.restore()
                break

    def unpack_packet(self, size: int):
        body = self.recv_buff.read(size)

        buff = self.buff_type(body)
        if self.bot.compression_threshold >= 0:
            uncompressed_length = VarInt.decode(buff)
            if uncompressed_length > 0:
                body = zlib.decompress(buff.read())
                buff = self.buff_type(body)

        return buff

    def pack_packet(self, data: bytes):
        if self.bot.compression_threshold >= 0:
            if len(data) >= self.bot.compression_threshold:
                data = VarInt.encode(len(data)) + zlib.compress(data)
            else:
                data = VarInt.encode(0) + data

        return VarInt.encode(len(data)) + data

    def send_packet(self, name: str, *data: tuple[bytes]):
        data: bytes = b"".join(data)
        data = VarInt.encode(self.packet_encoder.encoder[name]) + data

        data = self.pack_packet(data)

        data = self.cipher.encrypt(data)

        self.bot.transport.write(data)

    def __call__(self):
        self.logger.debug(f'Run protocol "{self.__class__.__name__}" state {self.state}')
        self.run()
        return self

    def __getattr__(self, name: str):
        router = self._table_routers_names.get(name)
        if router is None:
            for key, router in self._table_routers.items():
                if name == key.__name__.lower():
                    self._table_routers_names[name] = router
                    break
            else:
                raise AttributeError(f"'{self.__class__.__name__}' object has no attribute '{name}'")
        return router

==> bot4-0.0.11-py3-none-any.whl: bot4/protocol/protocolbase.py <==
import asyncio, zlib
from typing import Callable
from bot4.mcd.packets import Coder
from bot4.router.packetsrouter import PacketRouter, PacketHeandler, PacketStorage
from bot4.router.baserouter import BaseRouter, BaseHeandler
from bot4.buffer import BufferUnderrun, Buffer
from bot4.bot import botbase
from bot4.typesdata.v754 import VarInt
from bot4.exceptions import ProtocolError


def _add_callback(func_add: Callable):
    def _add(self: 'PortocolBase', event: BaseHeandler | str, func: Callable = None):
        def _w(func):
            if isinstance(event, str):
                getattr(self.router_packets, func_add.__name__)(event, func)
            elif isinstance(event, BaseHeandler):
                try:
                    getattr(self._table_routers[event.target_router], func_add.__name__)(event, func)
                except KeyError:
                    raise ProtocolError(f'router <{event.target_router.__name__}> unregistered')

        if func is None:
            return _w
        else:
            _w(func)
    return _add


class PortocolBase(asyncio.Protocol):
    router_packets: PacketRouter = None
    state: int
    unheandler = None

    def __init__(self, bot: 'botbase.BotBase',
                 packet_encoder_s: Coder,
                 router_packets: PacketRouter,
                 state: int):
        self.bot = bot
        
        self.loop = bot.loop
        self.recv_buff = bot.recv_buff
        self.cipher = bot.cipher
        self.buff_type = bot.buff_type
        self.logger = bot.logger
        
        self.state = state
        self.router_packets = router_packets
        self.packet_encoder = packet_encoder_s
        self._table_routers = {router_packets.__class__: router_packets}

        self._table_routers_names = {}
        self.table_routers_clear = self._table_routers.clear

    @_add_callback
    def on(self, event: BaseHeandler | str, func: Callable = None):
        ...

    @_add_callback
    def once(self, event: BaseHeandler | str, func: Callable = None):
        ...

    def reg_router(self, router: BaseRouter):
        if router.__class__ in self._table_routers:
            raise ProtocolError(f'router "{router.__class__.__name__}" is already registered')
        elif isinstance(router, BaseRouter):
            if self.router_packets.__class__ is router.__class__:
                self.router_packets = router
                # raise ProtocolError(f'router type <{router.__class__}> is not router_packets type <{self.router_packets.__class__}>')
            self._table_routers[router.__class__] = router
        else:
            raise ProtocolError(f'"router" <{router}> does not inherit "BaseRouter"')

    def reg_routers(self, *routers: BaseRouter):
        for r in routers:
            self.reg_router(r)

    def unreg_router(self, router: BaseRouter):
        if router.__class__ in self._table_routers:
            del self._table_routers[router.__class__]
            name = router.__class__.__name__.lower()
            if name in self._table_routers_names:
                del self._table_routers_names[name]

            if not self._table_routers:
                self.router_packets.clear()
                self.router_packets = None
            return True
        else:
            return False
    
    def run(self):
        self.bot.is_close = False
        # self.bot.state = self.state

    def switch(self):
        self.logger.debug(f'Switched protocol "{self.__class__.__name__}" state {self.state}')
        self.router_packets.clear()

    def connection_made(self, transport: asyncio.Transport) -> None:
        self.bot.transport = transport
        self.logger.info(f'Client connected to "{self.bot.ip}:{self.bot.port}"')

    async def asend_packet(self, *args):
        self.send_packet(*args)

    def connection_lost(self, exc: Exception | None) -> None:
        self.switch()
        self.bot._connection_lost(exc)

    def data_received(self, data: bytearray):
        self.recv_buff.add(self.cipher.decrypt(bytes(data)))
        
        if self.state != self.bot.state: self.logger.debug(f'{self.state} {self.bot.state}')
        while self.state == self.bot.state:
            self.recv_buff.save()
            try:
                buff: Buffer = self.unpack_packet(VarInt.decode(self.recv_buff))
                id_packet = VarInt.decode(buff)
                try:
                    self.router_packets.rout(id_packet, buff)
                except KeyError as e:
                    self.logger.critical(f'{e} state {self.state} protocol {self.__class__.__name__}')
            
            except BufferUnderrun:
                self.recv_buff.restore()
                break

    def unpack_packet(self, size: int):
        body = self.recv_buff.read(size)

        buff = self.buff_type(body)
        if self.bot.compression_threshold >= 0:
            uncompressed_length = VarInt.decode(buff)
            if uncompressed_length > 0:
                body = zlib.decompress(buff.read())
                buff = self.buff_type(body)

        return buff

    def pack_packet(self, data: bytes):
        if self.bot.compression_threshold >= 0:
            if len(data) >= self.bot.compression_threshold:
                data = VarInt.encode(len(data)) + zlib.compress(data)
            else:
                data = VarInt.encode(0) + data

        return VarInt.encode(len(data)) + data

    def send_packet(self, name: str, *data: tuple[bytes]):
        data: bytes = b"".join(data)
        data = VarInt.encode(self.packet_encoder.encoder[name]) + data

        data = self.pack_packet(data)

        data = self.cipher.encrypt(data)

        self.bot.transport.write(data)

    def __call__(self):
        self.logger.debug(f'Run protocol "{self.__class__.__name__}" state {self.state}')
        self.run()
        return self

    def __getattr__(self, name: str):
        router = self._table_routers_names.get(name)
        if router is None:
            for key, router in self._table_routers.items():
                if name == key.__name__.lower():
                    self._table_routers_names[name] = router
                    break
            else:
                raise AttributeError(f"'{self.__class__.__name__}' object has no attribute '{name}'")
        return router
